package com.xhkjedu.smarking.service.papercheck;

import com.xhkjedu.exception.ServiceException;
import com.xhkjedu.smarking.mapper.paper.MsPaperMapper;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckMapper;
import com.xhkjedu.smarking.mapper.papercheck.MsPaperCheckTeacherTaskMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentBlockMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentQuestionMapper;
import com.xhkjedu.smarking.mapper.system.UserMapper;
import com.xhkjedu.smarking.model.paper.MsPaperBlockQuestion;
import com.xhkjedu.smarking.model.papercheck.MsPaperCheck;
import com.xhkjedu.smarking.model.papercheck.MsPaperCheckTeacher;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentBlock;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentQuestion;
import com.xhkjedu.smarking.utils.MarkingUtil;
import com.xhkjedu.smarking.vo.papercheck.MsCheckParam;
import com.xhkjedu.smarking.vo.papercheck.MsPaperCheckVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.vo.system.UserVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description 阅卷批阅教师任务表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperCheckTeacherTaskService {
    @Resource
    private MsPaperCheckTeacherTaskMapper msPaperCheckTeacherTaskMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private MsPaperCheckMapper msPaperCheckMapper;
    @Resource
    private MsPaperMapper msPaperMapper;
    @Resource
    private MsPaperStudentBlockMapper msPaperStudentBlockMapper;
    @Resource
    private MsPaperStudentQuestionMapper msPaperStudentQuestionMapper;


    /**
     * @Description 阅卷进度-按题块
     * @Date 2024/10/30 10:23
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByBlock(Integer mpid) {
        return msPaperCheckTeacherTaskMapper.listByBlock(mpid);
    }

    /**
     * @Description 阅卷进度-按题块-查看详情
     * @Date 2024/10/30 10:44
     * @Author YWX
     * @Param [mblockid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> detailByBlock(Integer mpid, Integer mblockid) {
        MsPaperCheck check = msPaperCheckMapper.findByMBlockId(mpid, mblockid);
        if (check == null) {
            throw new ServiceException("该题块不存在");
        }
        Integer dispenseway = check.getDispenseway();
        List<Map> list = msPaperCheckTeacherTaskMapper.detailByBlock(mpid, mblockid, dispenseway, check.getChecktype());
        return setListNum(dispenseway, list);
    }

    //计算题块/试题的总数和未批阅的数量
    private List<Map> setListNum(Integer dispenseway, List<Map> list) {
        if (dispenseway.equals(2)) {
            int totalNum = list.stream().mapToInt(m -> N_Utils.obj2Int(m.get("totalNum"))).sum();
            int totalCheckedNum = list.stream().mapToInt(m -> N_Utils.obj2Int(m.get("checkedNum"))).sum();
            int uncheckedNum = totalNum - totalCheckedNum;
            for (Map m : list) {
                m.put("totalNum", totalNum);
                m.put("uncheckedNum", uncheckedNum);
            }
        }
        return list;
    }

    /**
     * @Description 阅卷进度-按教师
     * @Date 2024/10/30 10:52
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByTeacher(Integer mpid) {
        Integer correcttype = msPaperMapper.getCorrectTypeByMpId(mpid);
        if (!N_Utils.isTrueInteger(correcttype)) {
            throw new ServiceException("该试卷未设置批阅方式");
        }
        return msPaperCheckTeacherTaskMapper.listByTeacher(mpid, correcttype);
    }

    /**
     * @Description 教师阅卷-待阅卷列表
     * @Date 2024/10/30 14:41
     * @Author YWX
     * @Param [handleid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listPending(MsCheckParam param) {
        return msPaperCheckTeacherTaskMapper.listPending(param);
    }

    //设置查询参数
    public void setParam(MsCheckParam param) {
        UserVo vo = userMapper.getUserInfoForHandleLog(param.getHandleid());
        param.setSchoolid(vo.getSchoolid());
        param.setUsertype(vo.getUsertype());
    }

    /**
     * @Description 教师阅卷-待阅卷详情
     * @Date 2024/10/30 16:02
     * @Author YWX
     * @Param [param]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listPendingBlock(MsCheckParam param) {
        setParam(param);
        return msPaperCheckTeacherTaskMapper.listPendingBlock(param);
    }

    /**
     * @Description 教师阅卷-题块列表
     * @Date 2024/10/31 9:49
     * @Author YWX
     * @Param [param]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listTeacherBlock(MsCheckParam param) {
        setParam(param);
        return msPaperCheckTeacherTaskMapper.listTeacherBlock(param);
    }

    /**
     * @Description 获取待批题块
     * @Date 2024/10/31 11:01
     * @Author YWX
     * @Param [param]
     * @Return java.util.Map
     **/
    public ResultVo getCorrectBlock(MsCheckParam param) {
        Integer correcttype = msPaperMapper.getCorrectTypeByMpId(param.getMpid());
        if (!N_Utils.isTrueInteger(correcttype)) {
            throw new ServiceException("该试卷未设置批阅方式");
        }
        MsPaperCheckVo sq = null;
        Integer dispenseway = null;
        Integer type = null;//评卷方式1单评2双评
        Integer handleid = param.getHandleid();
        MsPaperCheck paperCheck = null;
        if (correcttype.equals(3)) {//按题块批阅
            List<MsPaperCheck> checks = msPaperCheckTeacherTaskMapper.listBlockCheck(param);
            if (N_Utils.isListEmpty(checks)) throw new ServiceException("暂无要批阅的题块");
            for (MsPaperCheck check : checks) {
                check.setTeacherid(handleid);
                sq = msPaperCheckTeacherTaskMapper.getCorrectBlock(check);
                if (sq != null) {
                    paperCheck = check;
                    break;
                }
            }
        } else if (correcttype.equals(2)) {//按试题批阅
            List<MsPaperCheck> checks = msPaperCheckTeacherTaskMapper.listQuestionCheck(param);
            if (N_Utils.isListEmpty(checks)) throw new ServiceException("暂无要批阅的试题");
            for (MsPaperCheck check : checks) {
                check.setTeacherid(handleid);
                sq = msPaperCheckTeacherTaskMapper.getCorrectQuestion(check);
                if (sq != null) {
                    paperCheck = check;
                    break;
                }
            }
        } else {//按班级批阅
            List<MsPaperCheck> checks = msPaperCheckTeacherTaskMapper.listClassCheck(param);
            if (N_Utils.isListEmpty(checks)) throw new ServiceException("暂无要批阅的班级");
            for (MsPaperCheck check : checks) {
                check.setTeacherid(handleid);
                sq = msPaperCheckTeacherTaskMapper.getCorrectClassQuestion(check);
                if (sq != null) {
                    paperCheck = check;
                    break;
                }
            }
        }

        if (paperCheck != null) {
            dispenseway = paperCheck.getDispenseway();
            type = paperCheck.getChecktype();
        }
        Integer totalNum = 0;
        Integer checkedNum = 0;
        if (sq != null) {
            MsPaperCheckTeacher teacher = msPaperCheckTeacherTaskMapper.getCheckNum(paperCheck);
            totalNum = teacher.getTasknum();
            checkedNum = teacher.getCheckednum();
            if (!dispenseway.equals(2)) {
                //更新教师阅卷任务状态
                msPaperCheckTeacherTaskMapper.updateCheckStatus(sq.getMpid(), sq.getMpsbid(), handleid, 2);
            }
            Integer checktype = sq.getChecktype();
            Integer firstcid = sq.getFirstcid();
            if (!N_Utils.isTrueInteger(checktype)) {
                if (type.equals(1)) {
                    checktype = 1;
                } else if (N_Utils.isTrueInteger(firstcid) && !handleid.equals(firstcid)) {
                    checktype = 2;
                } else {
                    checktype = 1;
                }
            }
            if (correcttype.equals(3)) {//按题块批阅
                MsPaperStudentBlock block = new MsPaperStudentBlock();
                block.setMpsbid(sq.getMpsbid());
                if (checktype.equals(1)) {
                    block.setFirstcid(handleid);
                } else {
                    block.setSecondcid(handleid);
                }
                block.setChecked(1);
                msPaperStudentBlockMapper.updateByPrimaryKeySelective(block);
            } else {//按试题/班级批阅
                MsPaperStudentQuestion question = new MsPaperStudentQuestion();
                question.setMpsqid(sq.getMpsbid());
                question.setChecked(1);
                if (checktype.equals(1)) {
                    question.setFirstcid(handleid);
                } else {
                    question.setSecondcid(handleid);
                }
                msPaperStudentQuestionMapper.updateByPrimaryKeySelective(question);
            }
            sq.setChecktype(checktype);
            sq.setMpid(paperCheck.getMpid());
        }
        Map map = new HashMap();
        map.put("sq", sq);
        map.put("totalNum", totalNum);
        map.put("checkedNum", checkedNum);
        String msg = "获取成功";
        return new ResultVo(0, msg, map);
    }

    /**
     * @Description 阅卷进度-按试题
     * @Date 2024/11/8 16:18
     * @Author YWX
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listByQuestion(Integer mpid) {
        Integer correcttype = msPaperMapper.getCorrectTypeByMpId(mpid);
        if (!N_Utils.isTrueInteger(correcttype)) {
            throw new ServiceException("该试卷未设置批阅方式");
        }
        return msPaperCheckTeacherTaskMapper.listByQuestion(mpid, correcttype);
    }

    /**
     * @Description 阅卷进度-按试题-查看详情
     * @Date 2024/11/11 9:24
     * @Author YWX
     * @Param [mpid, mblockid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> detailByQuestion(Integer mpid, Integer mblockid) {
        MsPaperCheck check = msPaperCheckMapper.findByMBlockId(mpid, mblockid);
        if (check == null) {
            throw new ServiceException("该试题不存在");
        }
        Integer dispenseway = check.getDispenseway();
        List<Map> list = msPaperCheckTeacherTaskMapper.detailByQuestion(mpid, mblockid, dispenseway, check.getChecktype());
        return setListNum(dispenseway, list);
    }

    /**
     * @Description 批阅
     * @Date 2024/11/12 10:02
     * @Author YWX
     * @Param [param]
     * @Return void
     **/
    public void correct(MsPaperCheckVo param) {
        MsPaperCheck check = msPaperCheckMapper.findByMBlockId(param.getMpid(), param.getMblockid());
        if (check == null) {
            throw new ServiceException("该阅卷任务不存在");
        } else if (check.getCheckstate().equals(0)) {
            throw new ServiceException("该阅卷任务未开始");
        } else if (check.getCheckstate().equals(2)) {
            throw new ServiceException("该阅卷任务已暂停");
        }
        Integer checktype = check.getChecktype();
        Integer correcttype = check.getCorrecttype();
        Integer mpsbid = param.getMpsbid();
        Integer hasgood = param.getHasgood();
        Integer hasbad = param.getHasbad();
        Integer hasproblem = param.getHasproblem();
        Integer problemtype = param.getProblemtype();
        String problemcomm = param.getProblemcomm();
        Double qscore = param.getQscore();
        List<MsPaperBlockQuestion> questions = param.getQuestions();
        int checked = 1;
        if (checktype.equals(1)) {//单评
            if (hasproblem.equals(0)) {
                checked = 2;
            }
            Double firstccore = param.getFirstccore();
            if (correcttype.equals(3)) {//按题块批阅
                //题块批阅信息
                MsPaperStudentBlock block = new MsPaperStudentBlock();
                block.setMpsbid(mpsbid);
                block.setChecked(checked);
                block.setFirstcime(N_Utils.getSecondTimestamp());
                block.setFirstccore(firstccore);
                block.setHasgood(hasgood);
                block.setHasbad(hasbad);
                block.setHasproblem(hasproblem);
                block.setProblemtype(problemtype);
                block.setProblemcomm(problemcomm);
                msPaperStudentBlockMapper.updateByPrimaryKeySelective(block);
                block.setStudentid(param.getStudentid());
                block.setMblockid(param.getMblockid());

                List<MsPaperStudentQuestion> studentQuestions = new ArrayList<>();
                for (MsPaperBlockQuestion question : questions) {
                    Double stuscore = question.getStuscore();
                    if (N_Utils.isTrueInteger(question.getMptqid())) {
                        MsPaperStudentQuestion studentQuestion = new MsPaperStudentQuestion();
                        studentQuestion.setMptqid(question.getMptqid());
                        studentQuestion.setFirstccore(stuscore);
                        studentQuestion.setStuscore(stuscore);
                        studentQuestions.add(studentQuestion);
                    } else {
                        String[] mptqids = question.getMergeqid().split(",");
                        String[] mergescores = question.getMergescore().split(",");
                        double[] scores = new double[mergescores.length];
                        int last = mergescores.length - 1;
                        double score = 0;
                        for (int i = 0; i < mergescores.length; i++) {
                            scores[i] = MarkingUtil.mul(MarkingUtil.div(mergescores[i], qscore), stuscore);
                            score = MarkingUtil.add(score, scores[i]);
                            if (i == last) {
                                scores[i] = MarkingUtil.sub(stuscore, score);
                            }
                        }
                        for (int i = 0; i < mptqids.length; i++) {
                            MsPaperStudentQuestion studentQuestion = new MsPaperStudentQuestion();
                            studentQuestion.setMptqid(N_Utils.obj2Int(mptqids[i]));
                            studentQuestion.setFirstccore(scores[i]);
                            studentQuestion.setStuscore(scores[i]);
                            studentQuestions.add(studentQuestion);
                        }
                    }
                }
                msPaperStudentQuestionMapper.checkByMptqidAndStudentId(studentQuestions, block);//更新学生单题批阅信息
            } else {//按试题批阅
                MsPaperStudentQuestion question = new MsPaperStudentQuestion();
                question.setMpsqid(mpsbid);
                question.setChecked(checked);
                question.setFirstcime(N_Utils.getSecondTimestamp());
                question.setFirstccore(firstccore);
                question.setHasgood(hasgood);
                question.setHasbad(hasbad);
                question.setHasproblem(hasproblem);
                question.setProblemtype(problemtype);
                question.setProblemcomm(problemcomm);
                msPaperStudentQuestionMapper.updateByPrimaryKeySelective(question);
            }
        }
        msPaperCheckTeacherTaskMapper.correct(param);
    }
}
